# sb::Admin::Entry
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Entry;
use strict;
use sb::Utils ();
use sb::Net::Ping ();
use sb::Admin::List ();
use base qw( sb::Admin::List );
use vars qw( $VERSION );
$VERSION = '0.36';
# 0.36 [2009/02/18] modified _build_files to use new API of sb::Build
# 0.35 [2009/02/10] modified _display to get editor properly
# 0.34 [2009/02/08] modified _display to use sb->get->plugin instead of sb::Plugin->get
# 0.33 [2009/01/29] modified _display to display valid modified date only
# 0.32 [2009/01/24] modified _display to display modified date
# 0.31 [2009/01/24] modified title_name to handle no title entry
# 0.30 [2008/10/22] modified get_as_tag to get a correct path of the small image
# 0.29 [2008/06/01] modified _update to record related categories correctly
# 0.28 [2007/11/30] modified _update to record modified date
# 0.27 [2007/11/29] enabled sb::Net::Ping
# 0.26 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.25 [2007/11/05] added title_name
# 0.24 [2007/10/01] modified _update to empty body and more
# 0.23 [2007/09/20] 'ents' value is not updated correctly, fixed _get_neighbor
# 0.22 [2007/09/14] ported from Serene Bach 2.*
# 0.21 [2007/05/04] changed _save_entry to check user 'imagetag' option uploading a file
# 0.20 [2007/02/15] changed _save_entry to output error message correctly
# 0.19 [2007/02/15] changed _save_entry to fix a bug
# 0.18 [2007/02/14] changed _save_entry to output ping error
# 0.17 [2006/02/16] changed _save_entry to update entry correctly
# 0.16 [2006/02/15] changed _save_entry to update entry correctly to avoid over-writing.
# 0.15 [2006/02/03] changed _save_entry to remain correct status of entry after uploading an image
# 0.14 [2006/02/01] changed _save_entry to check trackbak url and change sending trackabck timing properly
# 0.13 [2005/12/09] changed image_selector to display a list correctly
# 0.12 [2005/10/19] chnaged _clip_for_trackback/message to display list correctly
# 0.11 [2005/09/28] changed _save_entry to send trackback ping correctly
# 0.10 [2005/07/28] changed _save_entry to add text uploading file correctly
# 0.09 [2005/07/25] changed _open_entry to change opening related category field
# 0.08 [2005/07/17] changed _save_entry to change handling related categories
# 0.07 [2005/07/16] changed _build_files to change the order of building files
# 0.06 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.05 [2005/07/08] changed _save_entry to update "ping" correctly
# 0.04 [2005/06/29] changed image_selector to add new option
# 0.03 [2005/06/08] changed _open_entry to handle category addition form
# 0.02 [2005/06/07] changed _open_entry to implement bookmarklet
# 0.01 [2005/06/01] update instance variable after creating category
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'entry.html' };
sub JSFILE_NAME  (){ 'script/entryeditor.js' }
sub OPTION_STR   (){ '<option value="%s"%s>%s</option>' };
sub SEPARATOR    (){ '<!--more-->' };
sub DENIED_CHECK (){ '-' };
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	if ($self->mode eq 'new' or $self->mode eq 'edit')
	{
		my $editor = $self->get_editor();
		return( $self->SUPER::js_paths, $self->static_path . JSFILE_NAME, $editor->javascripts() );
	}
	return $self->SUPER::js_paths;
}
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	if ( $self->{'entry'} )
	{
		$title = sb->string(' &gt; Edit article');
		$title .= sb->string(' &gt; ');
		if ( $self->{'entry'}->subj() ne '' )
		{
			$title .= sb::Text->entitize($self->{'entry'}->subj());
		}
		else
		{
			$title .= sb->string('[no title]');
		}
	}
	elsif ($self->mode eq 'new')
	{
		$title = sb->string(' &gt; New article');
	}
	return $title;
}
# --------------------------------------
# // public functions - for sub class
sub timezone_selector
{
	my $self = shift;
	my %param = (
		'temp'    => undef,
		'tag'     => undef,
		'current' => $self->conf->value('conf_timezone'),
		@_
	);
	return if ( !defined($param{'temp'}) or !defined($param{'tag'}) );
	my $temp = $param{'temp'};
	my ($hour,$min) = ( $param{'current'} =~ /([\+-]\d\d)(\d\d)/ );
	my $sel_hour = '';
	foreach my $tz_hour ( @{$self->conf->value('setup_tz_hour')} )
	{
		$sel_hour .= sprintf(OPTION_STR,$tz_hour,($hour eq $tz_hour) ? ' selected="selected"' : '','UTC ' . $tz_hour);
	}
	my $sel_min = '';
	foreach my $tz_min ( @{$self->conf->value('setup_tz_min')} )
	{
		$sel_min .= sprintf(OPTION_STR,$tz_min,($min eq $tz_min) ? ' selected="selected"' : '',$tz_min);
	}
	$temp->num(0);
	$temp->tag($param{'tag'} . '_hour'=>$sel_hour);
	$temp->tag($param{'tag'} . '_min'=>$sel_min);
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	# --- variable declaration
	$self->{'cat'}   = undef;
	$self->{'ents'}  = undef;
	$self->{'blog'}  = undef;
	$self->{'entry'} = undef;
	$self->_initialize();
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	# --- initialize variables
	$self->_initialize(); # to make sure variables are set
	my %date = map {
		$_ => $self->cgi->value('entry_date_' . $_)
	} ('year','mon','day','hour','min','sec');
	$date{'zone'} = $self->cgi->value('entry_tz_hour') . $self->cgi->value('entry_tz_min');
	my %var = (
		'message'   => undef,
		'entry'     => undef,
		'category'  => undef,
		'ping'      => [split(/\0/,$self->cgi->value('entry_ping'))],
		'tbping'    => [split(/\n/,$self->cgi->value('entry_tbping'))],
		'date'      => sb::Time->new(%date)->data(),
		'olddate'   => undef,
		'entrybody' => '',
		'rewrite'   => ($self->cgi->value('id') ne '' and $self->cgi->value('id') != -1),
		'elem'      => {},
		@_
	);
	$var{'elem'} = {
		'subj' => sb::Text->new($self->cgi->value('entry_title'))->entitized(),
		'cat'  => ($self->cgi->value('entry_category') eq 'none') ? -1 : int($self->cgi->value('entry_category')),
		'auth' => ($self->cgi->value('entry_author') ne '') ? $self->cgi->value('entry_author') : $self->user->id,
		'stat' => ($self->cgi->value('open_save') ne '') ? 1 : 0,
		'file' => ($self->cgi->value('entry_file') =~ /^\w+$/) ? $self->cgi->value('entry_file') : '',
		'date' => $var{'date'},
		'tz'   => $date{'zone'},
		'add'  => '',
		'edit' => $self->user->id,
		'acm'  => ($self->cgi->value('entry_com') ne '')    ? $self->cgi->value('entry_com') : '',
		'atb'  => ($self->cgi->value('entry_tb') ne '')     ? $self->cgi->value('entry_tb') : '',
		'form' => ($self->cgi->value('entry_format') ne '') ? $self->cgi->value('entry_format') : '',
		'body' => ($self->cgi->value('entry_body') ne '')   ? $self->cgi->value('entry_body') : '',
		'more' => ($self->cgi->value('entry_more') ne '')   ? $self->cgi->value('entry_more') : '',
		'sum'  => sb::Text->new($self->cgi->value('entry_summary'))->entitized(),
		'key'  => sb::Text->new($self->cgi->value('entry_keyword'))->entitized(),
		'tmp'  => undef,
		'mod'  => $self->time->data,
	};
	if ($self->cgi->value('category_selected') ne '' or $self->user->get_option('auto_cat'))
	{
		my @related = split(/\0/,$self->cgi->value('category_selected'));
		if ($self->user->get_option('auto_cat') and $var{'elem'}->{'cat'} ne '')
		{
			my $child = $self->cat()->{$var{'elem'}->{'cat'}};
			push(@related,$child->main) if ($child and $child->main ne '');
		}
		if (@related)
		{
			@related = sb::Utils->remove_duplicates(@related);
			$var{'elem'}->{'add'} = ',' . join(',',@related) . ',';
		}
	}
	$var{'elem'}->{'subj'} =~ tr/\x0D\x0A//d;
	$var{'elem'}->{'key'}  =~ tr/\x0D\x0A//d;
	$var{'elem'}->{'tmp'} = join("\n",@{$var{'tbping'}});
	if ($self->user->get_option('sequel') != 0)
	{ 
		# when sequel option is 0, more field may be available.
		# otherwise we should check separator in body.
		my $separator = "\n" . SEPARATOR . "\n";
		my ($body,$more) = split(/$separator/,$var{'elem'}->{'body'},2);
		$var{'elem'}->{'body'} = $body;
		$var{'elem'}->{'more'} = $more;
	}
	$var{'elem'}->{'body'} = '' if ($var{'elem'}->{'body'} =~ /^\s+$/);
	$var{'elem'}->{'more'} = '' if ($var{'elem'}->{'more'} =~ /^\s+$/);
	$var{'entrybody'} = $var{'elem'}->{'body'} . $var{'elem'}->{'more'};
	if ($self->cgi->value('entry_add_category') ne '')
	{
		$var{'category'}->{'name'} = sb::Text->new($self->cgi->value('entry_add_category'))->entitized();
		$var{'category'}->{'name'} =~ tr/\x0D\x0A//d;
		$var{'category'}->{'sub'} = ($self->cgi->value('entry_add_sub') ne '') ? 1 : undef;
	}
	if ($var{'elem'}->{'cat'} ne '' and $self->cat()->{$var{'elem'}->{'cat'}})
	{
		$var{'elem'}->{'stat'} = 2 if ($var{'elem'}->{'stat'} and $self->cat()->{$var{'elem'}->{'cat'}}->get_option('top'));
	}
	if ($var{'rewrite'})
	{ # id is declared
		$var{'entry'} = sb::Data->load('Entry','id'=>$self->cgi->value('id'));
		$var{'elem'}->{'stat'} = $var{'entry'}->stat if ($var{'entry'} and ($self->cgi->value('upload') or $self->cgi->value('findtb')));
	}
	else
	{ # check the duplication
		$var{'entry'} = $self->_get_duplicated_entry(
			'subj' => $var{'elem'}->{'subj'},
			'body' => $var{'elem'}->{'body'},
			'more' => $var{'elem'}->{'more'},
		);
	}
	if ($var{'entry'})
	{ # existed article
		$var{'rewrite'} = 1;
		if ($var{'entry'}->date != $var{'elem'}->{'date'})
		{
			my ($prv,$nxt) = $self->_get_neighbor($var{'entry'});
			$var{'olddate'}->{'date'} = $var{'entry'}->date;
			$var{'olddate'}->{'zone'} = $var{'entry'}->tz;
			$var{'olddate'}->{'prev'} = $prv;
			$var{'olddate'}->{'next'} = $nxt;
		}
	}
	else
	{ # new article
		$var{'rewrite'} = undef;
		$var{'entry'} = sb::Data::Entry->create_new('id'=>-1); # create temporary entry
	}
	# we should have an entry object
	die(sb->string('Unknown error.') . "\n") if (!$var{'entry'});
	foreach my $elem ( keys(%{$var{'elem'}}) )
	{ # copy all elements
		next if (!$elem);
		$var{'entry'}->$elem($var{'elem'}->{$elem});
	}
	$self->{'entry'} = $var{'entry'} if ($var{'rewrite'});
	if ($self->cgi->value('upload'))
	{ # --- upload a file via entry editor
		my @files = $self->upload_file(
			'max'  => 1,
			'data' => 1,
			'over' => ($self->cgi->value('upload_overwrite') eq 'on'),
		);
		if (@files and !$self->user->get_option('imagetag'))
		{
			my $img = $files[0];
			my $target = $self->cgi->value('insert_target');
			my $option = ('file','thumb','link','link')[$self->user->get_option('imagelist')];
			$target = 'body' if ($target ne 'body' and $target ne 'more');
			$option = 'link' if (!$img->is_image);
			my $text = $var{'entry'}->$target() . $img->get_as_tag('type'=>'link');
			$var{'entry'}->$target($text);
		}
		return $self->_display(
			'message'  => ($#files > -1) 
				? sprintf(sb->string('Added %d item(s).<br />'),$#files + 1) 
				: sb->string('Failed to add item(s).'),
			'entry'    => $var{'entry'},
			'category' => $var{'category'},
		);
	}
	if ($self->cgi->value('findtb'))
	{ # --- try to detect trackback url
		my @urls = sb::Net::Ping->new->discover_trackback($var{'entrybody'});
		push(@{$var{'tbping'}},@urls) if (@urls);
		$var{'entry'}->tmp(join("\n",@{$var{'tbping'}}));
		return $self->_display(
			'message'  => sprintf(sb->string('Found %d trackback url(s).<br />'),$#urls + 1),
			'entry'    => $var{'entry'},
			'category' => $var{'category'},
		);
	}
	if ($var{'entrybody'} eq '')
	{ # --- article body must be entered.
		return $self->_display(
			'message'  => sb->string('No article body.'),
			'entry'    => $var{'entry'},
			'category' => $var{'category'},
		);
	}
	if ($var{'category'})
	{ # create new category
		my $category = $self->create_category(
			'main' => $var{'elem'}->{'cat'},
			'name' => $var{'category'}->{'name'},
			'sub'  => $var{'category'}->{'sub'},
			'num'  => 1,
		);
		if ($category)
		{
			$var{'category'} = undef;
			$var{'entry'}->cat($category->id);
			$self->cat()->{$category->id} = $category;
		}
	}
	if ($self->conf->value('basic_img_chck'))
	{ # --- file usage check
		$self->_check_image(
			'text' => $var{'entrybody'},
			'id'   => $var{'entry'}->id,
		);
	}
	if ($var{'entry'})
	{ # --- finally we can update created/modified article.
		if ($var{'entry'}->id == -1)
		{
			my $entry = sb::Data->add('Entry',%{$var{'entry'}});
			die(sb->string('Unknown error.') . "\n") if (!$entry);
			$var{'entry'} = $entry;
		}
		sb::Data->update($var{'entry'});
		$self->{'entry'} = $var{'entry'}; # now the created article becomes an existed article.
		push(@{$self->{'ents'}},$var{'entry'}) if (!$var{'rewrite'}); # we should update article buffer as well.
		$var{'message'} .= $self->_build_files($var{'olddate'});
	}
	if ($var{'entry'}->stat != 0)
	{ # post process for opened article.
		# --- send trackback pings
		if ($var{'entry'}->cat ne '')
		{ # add the trackback pings of category
			my $check = $self->cat()->{$var{'entry'}->cat};
			my $tburl = ($check) ? $check->url : '';
			my $sent = $var{'entry'}->ping;
			push(@{$var{'tbping'}},$tburl) if ($tburl ne '' and $sent !~ m!$tburl!);
		}
		my $ping_sender = sb::Net::Ping->new;
		my $stat = $ping_sender->send_trackback(
			'url'       => $var{'entry'}->permalink,
			'excerpt'   => $var{'entry'}->sum,
			'title'     => $var{'entry'}->subj,
			'blog_name' => $self->blog()->title,
			'list'      => $var{'tbping'},
			'eid'       => $var{'entry'}->id,
			'now'       => $self->{'time'},
		);
		if ($stat)
		{
			if (ref($stat->{'sent'}) eq 'ARRAY')
			{
				$var{'entry'}->add_ping(@{$stat->{'sent'}});
				foreach my $success ( @{$stat->{'sent'}} )
				{
					$var{'message'} .= sprintf(sb->string('Sent ping to %s.<br />'),$success) . "\n";
				}
			}
			if (ref($stat->{'error'}) eq 'ARRAY')
			{
				$var{'entry'}->tmp(join("\n",@{$stat->{'error'}}));
				foreach my $failure ( @{$stat->{'error'}} )
				{
					my $err_msg = sb::Text->new($ping_sender->error->{$failure})->entitized();
					$err_msg =~ tr/\x0D\x0A//d;
					$var{'message'} .= $failure . '[failed]' . $err_msg . "<br />\n";
				}
			}
			sb::Data->update($var{'entry'});
		}
		elsif (@{$var{'tbping'}} and $ping_sender)
		{
			$var{'message'} .= '[ping failed]' . $ping_sender->error . "<br />\n";
		}
		if (@{$var{'ping'}})
		{ # --- send update pings
			my $stat = sb::Net::Ping->new->send_update(
				'list' => $var{'ping'},
				'mode' => 'ping',
				'name' => $self->blog()->title,
			);
			if ($stat and ref($stat->{'sent'}) eq 'ARRAY')
			{
				foreach my $success ( @{$stat->{'sent'}} )
				{
					$var{'message'} .= sprintf(sb->string('Sent ping to %s.<br />'),$success) . "\n";
				}
			}
		}
	}
	$var{'message'} .= ($var{'rewrite'})
		? sb->string('Modified.<br />')
		: sb->string('Your request has been processed.');
	if ($self->cgi->value('bm'))
	{
		$var{'message'} .= ' <a href="#" onclick="window.close()">' . sb->string('[Close]') . '</a>';
	}
	return $self->_display(
		'message'  => $var{'message'},
		'entry'    => $var{'entry'},
		'category' => $var{'category'},
	);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message'  => undef,
		'entry'    => undef, # updated article data
		'newtext'  => undef, # default text
		'category' => undef, # category data
		@_
	);
	$self->_initialize(); # to make sure variables are set
	# --- set article data
	# [note] existed article : $self->{'entry'} is an instance of sb::Data::Entry
	# [note] new article     : $self->{'entry'} is undef
	if ($self->cgi->value('eid') =~ /^\d+$/)
	{
		$self->{'entry'} = sb::Data->load('Entry','id'=>$self->cgi->value('eid'));
	}
	$param{'entry'} = $self->{'entry'} if ($self->{'entry'});
	# --- define local variables
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $entry = $param{'entry'};
	die(sb->string('Not authorized.') . "\n") if ( $entry and !$self->has_permission('id'=>$entry->auth) );
	my %option = (
		'format'    => ($self->{'entry'}) ? $entry->form : $self->user->get_option('format'),
		'comment'   => ($self->{'entry'}) ? $entry->acm  : $self->user->get_option('comment'),
		'trackback' => ($self->{'entry'}) ? $entry->atb  : $self->user->get_option('trackback'),
		'sequel'    => $self->user->get_option('sequel'),
		'summary'   => ($self->user->get_option('advanced') == 1),
		'advanced'  => ($self->user->get_option('advanced') == 1),
		'tb_option' => ($self->user->get_option('tb_option') == 0),
		'cat_open'  => ($self->user->get_option('cat_open') == 0),
	);
	if ($self->{'entry'})
	{ # some options are changed depends on the article
		$option{'sequel'} = 2 if ($option{'sequel'} == 1 and $entry->more ne '');
		$option{'summary'} = ($entry->entitize('sum') ne '');
	}
	{ # --- categories
		$temp->tag('sb_entry_category'=>
			sprintf(OPTION_STR,'none','',sb->string('No category')) .
			$self->category_selector(
				'cat' => [ sort { $b->order <=> $a->order } values(%{$self->cat()}) ],
				'now' => ($entry) ? $entry->cat : $self->user->cat,
			)
		);
		if ($entry and $entry->add ne '')
		{ # additional categories
			my $options = '';
			foreach ( split(',',$entry->add) )
			{
				next if ($_ eq '');
				next if (!$self->cat->{$_});
				$options .= sprintf(OPTION_STR,$_,'',$self->cat->{$_}->name);
			}
			$temp->num(0);
			$temp->tag('sb_multi_category_current'=>$options);
			$temp->block('sb_extra_category'=>1) if ($option{'cat_open'});
		}
		if ($param{'category'})
		{ # buffered category
			$temp->num(0);
			$temp->tag('sb_entry_add_category'=>$param{'category'}->{'name'});
			$temp->tag('sb_entry_add_sub'=>'checked="checked"') if ($param{'category'}->{'sub'});
		}
		# adding category form (available for advanced user)
		$temp->block('add_category_form'=>1) if ($self->user->stat < 2);
	}
	if ( $self->{'entry'} )
	{ # --- artcile status and neighbor article navigator
		$temp->num(0);
		$temp->tag('sb_entry_status'=>$self->display_entry_status($entry));
		my @parts_list = (
			{ 'tag' => 'prev', 'src' => 'arrow_l.gif', 'alt' => '&lt;&lt;' },
			{ 'tag' => 'next', 'src' => 'arrow_r.gif', 'alt' => '&gt;&gt;' },
		);
		my $path = $self->script_path . '?__mode=edit&amp;eid=';
		foreach my $obj ( $self->_get_neighbor($entry) )
		{
			my $parts = shift(@parts_list);
			my $flag = ($obj and $self->cgi->value('bm') ne 'on');
			$temp->tag('sb_entry_' . $parts->{'tag'} =>
				sprintf('%s<img src="%s%s" width="20" height="10" title="%s" alt="%s" />%s',
					($flag) ? '<a href="' . $path . $obj->id . '">' : '', # begging of anchor tag
					$self->conf->value('srv_temp') . 'image/',            # common path
					($flag) ? $parts->{'src'} : 'blank.gif',              # image file name
					($flag) ? $obj->subj : '',                            # title attribute
					($flag) ? $parts->{'alt'} : '',                       # alt attribute
					($flag) ? '</a>' : ''                                 # end of anchor tag
				)
			);
		}
	}
	foreach my $key ('format','comment','trackback')
	{ # --- options for format / comment / trackback
		next if ($option{$key} !~ /^\d+$/);
		$self->select_option(
			'cms'      => $temp,
			'tag'      => 'sb_entry_' . $key . '_',
			'selected' => $option{$key},
		);
	}
	{ # addtional format
		my $selector = '';
		my @filters  = sb->get->plugin->text_filters;
		foreach my $name (@filters)
		{
			$selector .= sprintf(OPTION_STR,$name,($option{'format'} eq $name) ? ' selected="selected"' : '',$name);
		}
		$temp->tag('sb_entry_extra_format'=>$selector);
	}
	{ # --- date and time
		my $date = (!$self->{'entry'} or $self->user->get_option('init_date')) ? $self->time->data : $entry->date;
		my $zone = ($self->{'entry'}) ? $entry->tz : $self->conf->value('conf_timezone');
		$self->_generate_dateform(
			'temp'    => $temp,
			'tag'     => 'sb_entry',
			'current' => $date,
			'zone'    => $zone,
		);
		if ($option{'advanced'})
		{
			$self->timezone_selector(
				'temp'    => $temp,
				'tag'     => 'sb_entry_zone',
				'current' => $zone,
			);
		}
		else
		{
			my ($hour,$min) = ( $zone =~ /([\+-]\d\d)(\d\d)/ );
			my $hidden = '<input type="hidden" name="%s" value="%s" />';
			$temp->tag('hidden_entry_tz_hour'=>sprintf($hidden,'entry_tz_hour',$hour));
			$temp->tag('hidden_entry_tz_min' =>sprintf($hidden,'entry_tz_min' ,$min ));
		}
	}
	{ # --- update pings
		my @ping_list = split('\\n',$self->conf->value('conf_edit_ping'));
		my $flag = ($self->user->get_option('ping') and (!$self->{'entry'} or $entry->stat == 0));
		my $num = 0;
		foreach my $ping (@ping_list)
		{
			if ($num == 0)
			{
				$temp->num(0);
				$temp->tag('sb_entry_ping_name_one'=>$ping);
			}
			else
			{
				$temp->num($num - 1);
				$temp->tag('sb_entry_ping_name'=>$ping);
			}
			$temp->tag('sb_entry_ping'=>($flag) ? 'checked="checked"' : '');
			$num++;
		}
		$temp->num(0);
		$temp->tag('sb_enttry_ping_num'=>$num);
		$temp->block('sb_entry_ping'=>($num > 0) ? 1 : 0);
		$temp->block('sb_entry_ping_moreone'=>($num > 1) ? $num - 1 : 0);
	}
	{ # --- dislay settings
		$temp->num(0);
		$temp->block('sb_entry_sum_form'     => ($option{'summary'}) ? 1 : 0);
		$temp->block('sb_entry_advancedform' => ($option{'advanced'}) ? 1 : 0);
		$temp->block('sb_entry_tbform'       => ($option{'tb_option'}) ? 1 : 0);
		if ($self->conf->value('conf_entry_archive') eq 'Individual')
		{
			$temp->block('sb_entry_file' => ($option{'advanced'}) ? 1 : 0);
			$temp->tag('sb_entry_suffix' => $self->conf->value('basic_suffix'));
		}
	}
	if ($option{'advanced'} and $self->user->stat == 0)
	{ # --- authors (it's available for super user only
		my $current = ($self->{'entry'}) ? $entry->auth : $self->user->id;
		my $selector = '';
		my @array = sort { $a->id <=> $b->id } values(%{$self->users});
		foreach my $usr_chk ( @array )
		{
			$selector .= sprintf(OPTION_STR,$usr_chk->id,($usr_chk->id eq $current) ? ' selected="selected"' : '',$usr_chk->real);
		}
		$temp->num(0);
		$temp->tag('sb_entry_author'=>$selector);
		$temp->block('sb_entry_user'=>1);
	}
	if ($self->{'entry'} and $entry and $entry->ping ne '')
	{ # --- sent trackbacks
		my $ping = $entry->ping;
		$ping =~ s/\n/<br \/>/g;
		$temp->num(0);
		$temp->tag('sb_entry_tbsent'=>$ping);
		$temp->block('entry_sent_ping'=>1);
	}
	if ($entry and $entry->tmp ne '')
	{ # --- trackbacks to send
		$temp->num(0);
		$temp->tag('sb_entry_tbping'=>$entry->tmp);
	}
	if ($self->mode() eq 'bm' or $self->cgi->value('bm') eq 'on')
	{ # --- for Quickpost
		$temp->num(0);
		$temp->tag('sb_bm_para'=>'on');
		if ($self->cgi->value('bm') ne 'on')
		{
			my $orig = $self->cgi->value('_u');
			my $url  = _decode_uri($orig);
			$url = substr($url,0,index($url,'#')) if (index($url,'#') > -1);
			my @urls = (); # sb::Net::Ping->new->discover_trackback($url . ' ' . _decode_uri($orig)); # [todo]
			$temp->tag('sb_entry_tbping'=>join("\n",@urls)) if (@urls);
		}
	}
	if ($entry)
	{
		$temp->num(0);
		$temp->tag('sb_entry_title'   => $entry->subj);
		$temp->tag('sb_entry_summary' => $entry->sum) if ($entry->entitize('sum') ne '');
		$temp->tag('sb_entry_key'     => $entry->key);
		$temp->tag('sb_entry_file'    => $entry->file);
		if ($self->{'entry'})
		{
			$temp->tag('sb_edit_viewlink' => $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_sb') . '?eid=' . $entry->id);
			$temp->tag('sb_entry_id' => $entry->id);
			$temp->tag('sb_entry_modified_date' =>  sb::Time->format(
					'time' => $entry->mod() ? $entry->mod() : $entry->date(),
					'form' => '%YearShort%.%Mon%.%Day% %Hour%:%Min%',
					'zone' => $self->conf->value('conf_timezone'),
				));
			$temp->block('sb_edit_viewpage' => 1);
			$self->_display_list('temp'=>$temp,'id'=>$entry->id);
		}
	}
	{ # --- editors
		my $editor = $self->get_editor()->new( %$self, 'user' => $self->user );
		my $body = 
			  ($entry)                  ? $entry->entitize('body')
			: ($param{'newtext'} ne '') ? $param{'newtext'}
			:                             '';
		if ($entry and $option{'sequel'} == 2 and $entry->more ne '')
		{
			$body .= "\n" . SEPARATOR . "\n" . $entry->entitize('more');
		}
		$temp->tag('sb_body_editor' => $editor->display_editor(
			'id' => 'entry_body',
			'content' => $body,
			'addimage'  => 'body',
		));
		if ($option{'sequel'} == 0)
		{
			$temp->tag('sb_more_editor' => $editor->display_editor(
				'id' => 'entry_more',
				'content' => ($entry) ? $entry->entitize('more') : '',
				'addimage'  => 'more',
			));
			$temp->block('sb_entry_moreform'=>1);
		}
	}
	{ # --- file upload form
		$self->imagedir_selector(
			'cms'   => $temp,
			'tag'   => 'sb_img_dir',
			'thumb' => 'sb_edit_imagethumb',
			'over'  => 'check',
		);
	}
	$temp->block('sb_new_mode'=>1) if (!$self->{'entry'});
	$temp->block('sb_edit_mode'=>1) if ($self->{'entry'});
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - common
sub _initialize
{
	my $self = shift;
	$self->generate_accessor('cat','ents','blog');
	$self->cat({sb::Data->load_as_hash('Category')}) if (! defined($self->{'cat'}) );
	$self->ents([sb::Data->load('Entry','sort'=>'date','order'=>0)]) if (! defined($self->{'ents'}));
	$self->blog(sb::Data->load('Weblog','id'=>0)) if (! defined($self->{'blog'}))
}
sub _get_neighbor
{
	my $self  = shift;
	my $entry = shift;
	my $stat  = shift;
	return( undef, undef ) if (!$entry);
	return( undef, undef ) if (!$self->{'ents'} or !@{$self->{'ents'}});
	my ($prv,$nxt);
	my $check = -1;
	my @array = ();
	foreach my $check (@{$self->{'ents'}})
	{
		push(@array,$check) if (!$stat or $check->stat > 0);
	}
	for (my $i=0;$i<@array;$i++)
	{
		next if ($entry->id != $array[$i]->id);
		$check = $i;
		last;
	}
	$prv = $array[$check - 1] if ($check > 0);
	$nxt = $array[$check + 1] if ($check > -1 and $check < $#array);
	return( $prv,$nxt );
}
sub _decode_uri
{
	$_[0] =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2',$1)/eg;
	return $_[0];
}
# --------------------------------------
# // private functions - update
sub _build_files
{
	my $self = shift;
	my $old  = shift;
	return if (!$self->{'entry'});
	my $builder = sb::Build->new(
		'time'      => $self->time->data,
		'user'      => $self->users,
		'cat'       => $self->cat,
		'sortedcat' => [ sort { $b->order <=> $a->order } values(%{$self->cat}) ],
		'blog'      => $self->blog,
	);
	$builder->construct_entry(
		$self->{'entry'}, # $target
		[$self->_get_neighbor($self->{'entry'},'opened')], # $neighbor
		$old, # $oldstate
	);
}
sub _check_image
{
	my $self = shift;
	my %param = (
		'text' => undef,
		'id'   => undef,
		@_
	);
	return if ( !$param{'text'} or !defined($param{'id'}) );
	my @urls = sb::Utils->remove_duplicates(($param{'text'} =~ /<img.*?src="(.*?)"/ig));
	my @imgs = sb::Data->load('Image');
	my @update = ();
	my @ignore = ();
	foreach my $img (@imgs)
	{
		my $chk_image = $img->get_url();
		my $chk_thumb = $img->get_url('thumb');
		if (grep(/^(\Q$chk_image\E|\Q$chk_thumb\E)$/,@urls))
		{
			push(@update,$img);
		}
		elsif (index($img->eid,$param{'id'} . ':') > -1)
		{
			push(@ignore,$img);
		}
	}
	foreach my $img (@update)
	{
		my @buf = split(':',$img->eid);
		push(@buf,$param{'id'});
		$img->eid(join(':',sb::Utils->remove_duplicates(@buf)) . ':');
	}
	foreach my $img (@ignore)
	{
		my @buf = split(':',$img->eid);
		@buf = grep { $_ ne $param{'eid'} } @buf;
		$img->eid(join(':',@buf) . ':');
	}
	sb::Data->update(@update,@ignore);
}
sub _get_duplicated_entry
{
	my $self = shift;
	my %param = (
		'subj' => undef,
		'body' => undef,
		'more' => undef,
		@_
	);
	return sb::Data->load('Entry',
		'cond' => {
			'subj'=>$param{'subj'},
			'body'=>$param{'body'},
			'more'=>$param{'more'},
		},
		'num' => 1,
	);
}
# --------------------------------------
# // private functions - display
sub _generate_dateform
{
	my $self = shift;
	my %param = (
		'temp'    => undef,
		'tag'     => undef,
		'name'    => 'entry_date',
		'zone'    => $self->conf->value('conf_timezone'),
		'current' => undef,
		'format'  => {
			'date' => sb->string('Year%Year% Month%Mon% Day%Day%'),
			'time' => sb->string('%Hour%:%Min%:%Sec%'),
		},
		@_
	);
	return if (!defined($param{'temp'}) or !defined($param{'tag'}));
	my $temp = $param{'temp'};
	my $name = $param{'name'};
	foreach my $key ('Year','Mon','Day','Hour','Min','Sec')
	{
		my $type = ( grep(/$key/,('Year','Mon','Day')) ) ? 'date' : 'time';
		my $label = '%' . $key . '%';
		my $input = sprintf('<input type="text" class="text" name="%s" value="%s" size="%d" maxlength="%d" />',
			$name . '_' . lc($key),   # name
			$label,                   # value
			($key eq 'Year') ? 4 : 2, # size
			($key eq 'Year') ? 4 : 2  # maxlength
		);
		$param{'format'}{$type} =~ s/$label/$input/;
	}
	$temp->num(0);
	$temp->tag($param{'tag'} . '_date'=>
		sb::Time->format(
			'time'=>$param{'current'},
			'zone'=>$param{'zone'},
			'form'=>$param{'format'}{'date'}
		)
	);
	$temp->tag($param{'tag'} . '_time'=>
		sb::Time->format(
			'time'=>$param{'current'},
			'zone'=>$param{'zone'},
			'form'=>$param{'format'}{'time'}
		)
	);
}
sub _display_list
{
	my $self = shift;
	my %param = (
		'temp' => undef,
		'id'   => undef,
		@_
	);
	return if (!defined($param{'temp'}) or !defined($param{'id'}));
	my $temp = $param{'temp'};
	{ # --- comments
		my @message = sb::Data->load('Message','cond'=>{'eid'=>$param{'id'}});
		$temp->num(0);
		$temp->tag('sb_com_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_edit_com_list',
			'objects'  => \@message,
			'tags'     => {
				'sb_com_author' => sub { (shift)->display_message_author(@_) },
				'sb_com_date'   => 'date',
				'sb_com_stat'   => sub { (shift)->display_message_status(@_) },
				'sb_com_del'    => \&_display_checkbox,
			},
		);
	}
	{ # --- trackback
		my @trackback = sb::Data->load('Trackback','cond'=>{'eid'=>$param{'id'}});
		$temp->num(0);
		$temp->tag('sb_tb_num'=>sb::Data->matched);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_edit_tb_list',
			'objects'  => \@trackback,
			'tags'     => {
				'sb_tb_name' => sub { (shift)->display_message_author(@_) },
				'sb_tb_date' => 'date',
				'sb_tb_stat' => sub { (shift)->display_message_status(@_) },
				'sb_tb_del'  => \&_display_checkbox,
			},
		);
	}
	$temp->block('sb_edit_info'=>1);
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return DENIED_CHECK if (!$self->{'entry'});
	return ( $self->has_permission('id'=>$self->{'entry'}->auth) )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" class="form-control" />'
		: DENIED_CHECK;
}
1;
__END__
