# sb::Admin::Upload
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Upload;
use strict;
use sb::Admin::Entry ();
use base qw( sb::Admin::Entry );
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2009/01/24] reverted back to 0.09
# 0.10 [2008/11/13] modified _display to implement new feature (cancelled)
# 0.09 [2007/11/30] modified _update_change_names and _update_change_status
# 0.08 [2007/11/05] added title_name
# 0.07 [2007/09/17] ported from Serene Bach 2.*
# 0.06 [2006/09/30] changed _update_image to check user permission
# 0.05 [2005/07/27] chnaged _update_image to pass _open_entry correctly
# 0.04 [2005/07/22] changed _update_image to pass _display_image_list after deleting images
# 0.03 [2005/07/20] changed _is_editable to fix a bug
# 0.02 [2005/07/08] changed _update_image to build a js for cookie
# 0.01 [2005/06/07] changed _display_image_list to fix a bug
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'upload.html' };
sub THUMB_SIZE   (){ 80 };
sub TABLE_WIDTH  (){ 600 };
sub LIST_COLUMN  (){ 6 };
sub DENIED_CHECK (){ '-' };
sub NO_DATA      (){ '-' };
sub DATE_FORMAT  (){ '%YearShort%.%Mon%.%Day% %Hour%:%Min%' };
sub DATE_LANG    (){ 'en' };
sub LIST_MARK    (){ '[L]' };
sub COMICON_MARK (){ '[C]' };
sub TB_ICON_MARK (){ '[T]' };
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	$title = sb->string(' &gt; Upload');
	$title .= sb->string(' &gt; Multi-upload') if ($self->cgi->value('__type') eq 'form');
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'iid'} = undef;
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->regi() eq 'upload')
	{ # added new files
		$msg = $self->_update_upload_files();
	}
	elsif ($self->cgi->value('namechange') ne '')
	{ # changed the name of items
		$msg = $self->_update_change_names();
	}
	else
	{ # changed the status of selected items
		$msg = $self->_update_change_status();
	}
	return $self->SUPER::_display('newtext'=>$msg) if ($self->mode eq 'new');
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	$self->{'iid'} = $self->cgi->value('image_id') if ($self->cgi->value('image_id') =~ /^\d+$/);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $disptype = ( $self->cgi->value('__type') eq 'form' ) ? 'form' : 'main';
	if ($disptype eq 'main')
	{
		my $dispsort = ( $self->cgi->value('dispsort') ne '' ) ? $self->cgi->value('dispsort') : 'date';
		my $disp_num = $param{'setup'}->{'dispnum'} / 2;
		my $page = $self->get_current_page();
		my @imgs = sb::Data->load('Image',
			'sort'  => $dispsort,
			'order' => 1,
			'id'    => $self->{'iid'},
			'num'   => $disp_num,
			'bgn'   => $page * $disp_num,
			'cond'  => $self->_generate_condition,
		);
		$temp->num(0);
		if ($self->{'iid'} eq '')
		{
			$temp->tag('sb_list_page'=>$self->display_pagelink(
				'mode'    => 'upload',
				'column'  => LIST_COLUMN,
				'all'     => sb::Data->matched,
				'printed' => $#imgs + 1,
				'num'     => $disp_num,
				'params'  => ['dispsort','dispdir','dispdate','dispnum'],
			));
		}
		$self->imagedir_selector(
			'cms'    => $temp,
			'tag'    => 'sb_img_dirsel',
			'select' => '',
			'format' => '[%s]',
		);
		$self->imagedir_selector(
			'cms'    => $temp,
			'tag'    => 'sb_img_dispdir',
			'select' => $self->cgi->value('dispdir'),
		);
		$self->dispnum_selector(
			'cms'  => $temp,
			'now'  => $param{'setup'}->{'dispnum'},
			'half' => 'yes',
		);
		$self->monthly_selector(
			'cms'  => $temp,
			'tag'  => 'sb_message_dispdate',
			'data' => 'Image',
		);
		$self->select_option(
			'cms'      => $temp,
			'tag'      => 'sb_dispsort_',
			'selected' => $dispsort,
		);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_upload_list',
			'objects'  => \@imgs,
			'tags'     => {
				'sb_img_id'        => 'id',
				'sb_img_image'     => \&_display_image,
				'sb_img_nametext'  => 'name',
				'sb_img_disable'   => \&_is_editable,
				'sb_img_thumb'     => \&_thumb_mark,
				'sb_img_dirdisp'   => 'dir',
				'sb_img_imagsize'  => \&_display_image_size,
				'sb_img_statclass' => \&_display_status,
				'sb_img_sel'       => \&_display_checkbox,
				'sb_img_height'    => sub { THUMB_SIZE },
			},
		);
		if ($self->{'iid'} ne '' and $imgs[0])
		{
			my $date = sb::Time->format(
				'time' => $imgs[0]->date,
				'form' => DATE_FORMAT,
				'zone' => $imgs[0]->tz,
				'lang' => DATE_LANG,
			);
			my $author = $self->{'users'}->{$imgs[0]->auth};
			my $status = '';
			$status .= LIST_MARK if ($imgs[0]->stat eq '0');
			$status .= COMICON_MARK if ($imgs[0]->icon_c eq '1');
			$status .= TB_ICON_MARK if ($imgs[0]->icon_t eq '1');
			my $entry = ($imgs[0]->eid ne '') ? '' : NO_DATA;
			if ($imgs[0]->eid ne '')
			{
				my @eids = split(':',$imgs[0]->eid);
				foreach my $eid (@eids)
				{
					my $buf = sb::Data->load('Entry','id'=>$eid);
					$entry .= $self->linked_text(
						'text' => $buf->subj,
						'base' => '?__mode=edit&amp;eid=' . $buf->id,
						'user' => $buf->auth,
					) . '<br />';
				}
			}
			my $image = $imgs[0]->get_as_tag('max_w'=>TABLE_WIDTH,'max_h'=>TABLE_WIDTH,'type'=>'file');
			$temp->num(0);
			$temp->tag('sb_img_one_id'     => $imgs[0]->id);
			$temp->tag('sb_img_one_name'   => $imgs[0]->name);
			$temp->tag('sb_img_one_dir'    => $imgs[0]->dir);
			$temp->tag('sb_img_one_size'   => $self->_display_image_size($imgs[0]));
			$temp->tag('sb_img_one_author' => $author ? $author->real : NO_DATA);
			$temp->tag('sb_img_one_date'   => $date);
			$temp->tag('sb_img_one_status' => $status);
			$temp->tag('sb_img_one_entry'  => $entry);
			$temp->tag('sb_img_one_image'  => $image);
			$temp->tag('sb_img_one_type'   => $imgs[0]->get_content_type);
		}
	}
	if ($self->{'iid'} eq '')
	{
		my $max = ($disptype eq 'form') ? $self->conf->value('basic_max_img') : 1;
		for (my $i=0;$i<$max;$i++)
		{
			$temp->num($i);
			$temp->tag('sb_upload_num'=>$i);
			$temp->tag('sb_upload_dispnum'=>$i + 1) if ($disptype eq 'form');
		}
		$self->imagedir_selector(
			'cms'   => $temp,
			'tag'   => 'sb_img_dir',
			'thumb' => 'sb_upload_thumb',
			'over'  => 'check',
		);
		$temp->block('sb_upload_formname' => ($disptype eq 'form') ? 0 : 1 );
		$temp->block('sb_upload_eachform' => $max);
	}
	$temp->num(0);
	$temp->tag('sb_list_selall_status' => 'disabled="disabled"') if ($self->{'iid'} ne '');
	$temp->tag('sb_submenu_upload_main'=>' class="current"') if ($disptype eq 'main');
	$temp->tag('sb_submenu_upload_form'=>' class="current"') if ($disptype eq 'form');
	$temp->block('sb_upload_main'       => ($disptype eq 'main') ? 1 : 0);
	$temp->block('sb_upload_form'       => ($self->{'iid'} eq '') ? 1 : 0);
	$temp->block('sb_upload_select'     => ($self->{'iid'} eq '') ? 1 : 0);
	$temp->block('sb_upload_list_cond'  => ($self->{'iid'} eq '') ? 1 : 0);
	$temp->block('sb_upload_one'        => ($self->{'iid'} eq '') ? 0 : 1);
	$temp->block('sb_upload_one_status' => ($self->{'iid'} eq '') ? 0 : 1);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_upload_files
{
	my $self = shift;
	my @files = $self->upload_file(
		'data' => 1,
		'over' => ($self->cgi->value('upload_overwrite') eq 'on'),
	);
	return ($#files > -1)
		? sprintf(sb->string('Added %d item(s).<br />'),$#files + 1)
		: sb->string('Failed to add item(s).');
}
sub _update_change_names
{
	my $self = shift;
	my @ids = split("\0",$self->cgi->value('iid'));
	my @imgs = sb::Data->load('Image','cond'=>{'id'=>\@ids});
	foreach my $img (@imgs)
	{
		next if (!$self->has_permission('id'=>$img->auth));
		my $name = $self->cgi->value('img_name' . $img->id);
		$img->name(sb::Text->new($name)->entitized());
	}
	if ( $self->cgi->value('one_image') eq 'yes' 
	 and $#imgs == 0 )
	{
		$self->{'iid'} = $self->cgi->value('iid');
	}
	sb::Data->update(@imgs) if (@imgs);
	return sb->string('Modified.<br />');
}
sub _update_change_status
{
	my $self = shift;
	my $flag = undef;
	my @sels = split("\0",$self->cgi->value('sel'));
	my @imgs = sb::Data->load('Image','cond'=>{'id'=>\@sels});
	ACTION_SWITCH: {
		$_ = $self->cgi->value('regi_action');
		/^entry$/ && do
		{
			my $option = ('file','thumb','link','link')[$self->user->get_option('imagelist')];
			my $newtext = '';
			foreach my $img (@imgs)
			{
				$newtext .= $img->get_as_tag('type'=>$img->is_image ? $option : 'link') . "\n";
			}
			$self->mode('new');
			return $newtext;
		};
		/^del$/ && do
		{
			foreach my $img (@imgs)
			{
				$img->erase;
			}
			last ACTION_SWITCH;
		};
		/^lst(\d)$/ && do
		{
			my $new = $1;
			foreach my $img (@imgs)
			{
				$img->stat($new);
			}
			last ACTION_SWITCH;
		};
		/^com(\d)$/ && do
		{
			$flag = 1;
			my $new = $1;
			foreach my $img (@imgs)
			{
				$img->icon_c($new);
			}
			last ACTION_SWITCH;
		};
		/^tb(\d)$/ && do
		{
			my $new = $1;
			foreach my $img (@imgs)
			{
				$img->icon_t($new);
			}
			last ACTION_SWITCH;
		};
		/^\[(.+)\]$/ && do
		{
			my $new = $1;
			foreach my $img (@imgs)
			{
				$img->rename_file('dir'=>$new);
			}
			last ACTION_SWITCH;
		};
	} # end of ACTION_SWITCH
	if ( $self->cgi->value('one_image') eq 'yes' 
	 and $#imgs == 0 
	 and $self->cgi->value('regi_action') ne 'del')
	{
		$self->{'iid'} = $self->cgi->value('iid');
	}
	sb::Data->update(@imgs) if (@imgs);
	sb::Build->new()->build_cookie_js('force_to_create') if ($flag);
	return ($self->cgi->value('regi_action') eq 'del')
		? sprintf(sb->string('Deleted %d item(s).<br />'),$#imgs + 1)
		: sb->string('Modified.<br />');
}
# --------------------------------------
# // private functions - display
sub _generate_condition
{
	my $self = shift;
	my %cond = ();
	if ($self->cgi->value('dispdir') ne '')
	{
		$cond{'dir'} = $self->cgi->value('dispdir');
	}
	if ($self->cgi->value('dispdate') ne '')
	{
		$cond{'date'} = sb::Utils->create_date_condition($self->cgi->value('dispdate'));
		$cond{'__range'} = { 'date' => 'tz' };
	}
	return \%cond;
}
# --------------------------------------
# // private functions - list handlers
sub _display_image
{
	my $self = shift;
	my $obj  = shift;
	my $img = $obj->get_as_tag(
		'max_w' => THUMB_SIZE,
		'max_h' => THUMB_SIZE,
		'type'  => ($obj->thumb eq '') ? 'file' : 'thumb',
	);
	return $self->linked_text(
		'text' => $img,
		'base' => '?__mode=upload&amp;image_id=' . $obj->id,
		'user' => $obj->auth,
	);
}
sub _is_editable
{
	my $self = shift;
	my $obj  = shift;
	return ($self->has_permission('id'=>$obj->auth)) 
		? '' 
		: 'disabled="disabled"';
}
sub _thumb_mark
{
	my $self = shift;
	my $obj  = shift;
	return ($obj->thumb ne '') 
		? sb->string('[thumb]') 
		: '';
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return ( $self->has_permission('id'=>$obj->auth) and $self->{'iid'} eq '' )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" class="form-control" />'
		: DENIED_CHECK;
}
sub _display_image_size
{
	my $self = shift;
	my $obj  = shift;
	my ($w,$h) = $obj->get_size;
	return $w . ' x ' . $h . ' / ' . $obj->object('file')->size_with_unit();
}
sub _display_status
{
	my $self = shift;
	my $obj  = shift;
	my $text = '';
	$text .= 'lst' . $obj->stat;
	$text .= '_com' . $obj->icon_c;
	$text .= '_tb' . $obj->icon_t;
	$text .= '_[' . $obj->dir . ']';
	return $text;
}
1;
__END__
